!---------------------------------------------------------------------------------------
! mod_settings
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!> mod_settings
!!
!---------------------------------------------------------------------------------------


module mod_settings

  use mod_var

  implicit none
  private

  public :: files_t, config_t, read_settings, parse_string

  ! files_t contains information on paths and files

  type :: files_t

    character(len=max_path_len) :: path_output    ! path to output
    character(len=max_path_len) :: path_flexpart  ! path to flexpart output
    character(len=max_path_len) :: file_recept    ! receptor list file

  end type files_t

  ! config_t contains information on configuration settings

  type :: config_t
 
    integer                     :: datei           ! start date (yyyymmdd)
    integer                     :: datef           ! end date (yyyymmdd)
    logical                     :: lnested         ! use nested flexpart output (true or false)
    logical                     :: linversionout   ! inversion formatted flexpart output (true or false)

  end type config_t

  contains

    ! --------------------------------------------------
    ! check if input is numeric
    ! --------------------------------------------------

    logical function is_numeric(string)

      character(len=200), intent(in) :: string
      real :: var
      integer :: ierr

      read(string,*,iostat=ierr) var
      if(ierr.eq.0) then
        is_numeric=.true.
      else
        is_numeric=.false.
      endif

    end function is_numeric

    ! --------------------------------------------------
    ! check if input is logical
    ! --------------------------------------------------

    logical function is_logical(string)

      character(len=200), intent(in) :: string
      logical :: var
      integer :: ierr

      read(string,*,iostat=ierr) var
      if(ierr.eq.0) then
        is_logical=.true.
      else
        is_logical=.false.
      endif

    end function is_logical

    ! --------------------------------------------------
    ! split string
    ! --------------------------------------------------

    subroutine split(str,delims,before,sep)

    ! Routine finds the first instance of a character from 'delims' in the
    ! the string 'str'. The characters before the found delimiter are
    ! output in 'before'. The characters after the found delimiter are
    ! output in 'str'. The optional output character 'sep' contains the 
    ! found delimiter. A delimiter in 'str' is treated like an ordinary 
    ! character if it is preceded by a backslash (\). If the backslash 
    ! character is desired in 'str', then precede it with another backslash.

    character(len=*) :: str, delims, before
    character,optional :: sep
    logical :: pres
    character :: ch,cha
    integer :: i, k, ibsl, ipos, iposa, lenstr

    pres=present(sep)
    str=adjustl(str)
    lenstr=len_trim(str)
    if(lenstr == 0) return        ! string str is empty
    k=0
    ibsl=0                        ! backslash initially inactive
    before=' '
    do i=1,lenstr
      ch=str(i:i)
      if(ibsl == 1) then          ! backslash active
        k=k+1
        before(k:k)=ch
        ibsl=0
        cycle
      end if
      if(ch == '\') then          ! backslash with backslash inactive
        k=k+1
        before(k:k)=ch
        ibsl=1
        cycle
      end if
      ipos=index(delims,ch)
      if(ipos == 0) then          ! character is not a delimiter
        k=k+1
        before(k:k)=ch
        cycle
      end if
      if(ch /= ' ') then          ! character is a delimiter that is not a space
        str=str(i+1:)
        if(pres) sep=ch
        exit
      end if
      cha=str(i+1:i+1)            ! character is a space delimiter
      iposa=index(delims,cha)
      if(iposa > 0) then          ! next character is a delimiter
        str=str(i+2:)
        if(pres) sep=cha
        exit
      else
        str=str(i+1:)
        if(pres) sep=ch
        exit
      end if
    end do
    if(i >= lenstr) str=''
    str=trim(str)
    before=trim(before)
    return

    end subroutine split

    ! --------------------------------------------------
    ! routine to identify and read in content
    ! --------------------------------------------------

    subroutine read_content (line, identifier, cc, cn, cl, match)

      character (len=200), intent (in) :: line, identifier
      character (len=200), intent (out) :: cc ! character content
      real(kind=8),        intent (out) :: cn ! numeric content
      logical,             intent (out) :: cl ! logical content
      logical,             intent (out) :: match

      integer :: n

      n = len_trim (identifier)

      ! default: line does not match, cc="", cn=-9900.0
      cc    = ""
      cn    = -9999.9
      match = .false.
      if ( len_trim (line) >= n ) then
        if ( line(:n) == identifier(:n) ) then
          cc = adjustl (line(n+1:))
          if ( is_numeric(cc) ) then
            read (line(n+1:),*) cn
          else
            if( is_logical(cc) ) read (line(n+1:),*) cl
          endif
          match = .true.
        end if
      end if

    end subroutine read_content

    ! --------------------------------------------------
    ! routine to parse a string
    ! --------------------------------------------------

    subroutine parse_string (str, delims, args, nargs)

    ! Parses the string 'str' into arguments args(1), ..., args(nargs) based on
    ! the delimiters contained in the string 'delims'. Preceding a delimiter in
    ! 'str' by a backslash (\) makes this particular instance not a delimiter.
    ! The integer output variable nargs contains the number of arguments found.

    character(len=*) :: str, delims
    character(len=len_trim(str)) :: strsav
    character(len=*), dimension(:) :: args
    integer :: i, na, lenstr, nargs

    strsav=str
    na=size(args)

    do i=1,na
      args(i)=' '
    end do
    nargs=0
    lenstr=len_trim(str)
    if(lenstr==0) return

    do
      if(len_trim(str) == 0) exit
      nargs=nargs+1
      call split(str,delims,args(nargs))
    end do
    str=strsav

    end subroutine parse_string

    ! --------------------------------------------------
    ! read settings
    ! --------------------------------------------------

    subroutine read_settings(filename, files, config)

      character(len=200), intent(in)   :: filename
      type(files_t), intent(in out)    :: files
      type(config_t), intent(in out)   :: config

      integer            :: ierr, i, n
      character(len=200) :: line, identifier, cc
      real(kind=8)       :: cn
      logical            :: match, cl


     ! open file
      open (100, file = trim (filename), status = 'old', iostat=ierr)
      if(ierr.gt.0) then
        write (*,*) 'ERROR: cannot find '//trim(filename)
        stop
      endif

      ! loop over lines
      read_loop: do
        read (100, fmt='(A)', iostat=ierr) line

        ! first check if line exists or eof was reached
        if ( ierr.gt.0 ) then
          write (*,*) 'ERROR: check '//trim(filename)
          exit read_loop
        else if ( ierr.lt.0 ) then
          exit read_loop
        else

          ! omit comments ("#") and empty lines
          if ( index (trim (line), "#") .eq. 1 ) cycle read_loop
          if ( len   (trim (line))      .eq. 0 ) cycle read_loop

          ! read path settings
          identifier = "path_flexpart:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) files%path_flexpart = cc
          identifier = "path_output:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) files%path_output = cc
          identifier = "file_recept:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) files%file_recept = cc

          ! read config settings
          identifier = "datei:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) config%datei = cn
          identifier = "datef:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) config%datef = cn   
          identifier = "lnested:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) config%lnested = cl
          identifier = "linversionout:"
          call read_content (line, identifier, cc, cn, cl, match)
          if ( match ) config%linversionout = cl

        endif

      end do read_loop

    end subroutine read_settings

    ! --------------------------------------------------

end module mod_settings

